-- What the NEB module will use when it gets a host status event. 
-- It's nice and transaction safe, and its execution plan is helpfully cached after the first run.

CREATE or REPLACE FUNCTION update_host(
	text,boolean,boolean,boolean,boolean,
	boolean,boolean,boolean,boolean,text,
	int,boolean,boolean,boolean,int,
	int,text,int,text,float,
	float,int,int,int,int,
	int,boolean,text,int) RETURNS 
integer AS
'       DECLARE
		thisHostName			alias for $1;
		ActiveChecksEnabled		alias for $2;
		PassiveChecksEnabled		alias for $3;
		EventHandlerEnabled		alias for $4;
		FlapDetectionEnabled		alias for $5;
		NotificationsEnabled		alias for $6;
		FailurePredictionEnabled	alias for $7;
		ProcessPerformanceData		alias for $8;
		ObsessOverHost			alias for $9;
		EventHandler			alias for $10;
		ModifiedAttributes		alias for $11;
		HasBeenChecked			alias for $12;
		ShouldBeScheduled		alias for $13;
		IsFlapping			alias for $14;
		SoftState			alias for $15;
		HardState			alias for $16;
		PluginOutput			alias for $17;
		CurrentAttempt			alias for $18;
		CheckCommand			alias for $19;
		CheckExecuteTime		alias for $20;
		CheckLatency			alias for $21;
		LastCheck			alias for $22;
		NextCheck			alias for $23;
		MaxAttempts			alias for $24;
		LastNotification		alias for $25;
		NextNotification		alias for $26;
		NoMoreNotifications		alias for $27;
		PerformanceData			alias for $28;
		CheckType			alias for $29;
     		thisHostID			int;
		c				int;
		states				record;
        BEGIN
		-- get the host id. If this is a new host, insert the host and get the id
		select into thisHostID id FROM host WHERE name = thisHostName;

		if thisHostID is null then
			insert into host (name,
					  active_checks_enabled,
					  passive_checks_enabled,
					  event_handler_enabled,
					  flap_detection_enabled,
					  notifications_enabled,
					  failure_prediction_enabled,
					  process_performance_data,
					  obsess_over_host,
					  event_handler,
					  modified_attributes,
					  has_been_checked,
					  should_be_scheduled)
				values (thisHostName,
					ActiveChecksEnabled,
					PassiveChecksEnabled,
					EventHandlerEnabled,
					FlapDetectionEnabled,
					NotificationsEnabled,
					FailurePredictionEnabled,
					ProcessPerformanceData,
					ObsessOverHost,
					EventHandler,
					ModifiedAttributes,
					HasBeenChecked,
					ShouldBeScheduled);

			-- assume for the moment that this worked just fine
			thisHostID := currval(''public.host_id_seq'');
		else
			-- okay, the host is already here, so just update it
			update host set active_checks_enabled = ActiveChecksEnabled,
					passive_checks_enabled = PassiveChecksEnabled,
					event_handler_enabled = EventHandlerEnabled,
					flap_detection_enabled = FlapDetectionEnabled,
					notifications_enabled = NotificationsEnabled,
					failure_prediction_enabled = FailurePredictionEnabled,
					process_performance_data = ProcessPerformanceData,
					obsess_over_host = ObsessOverHost,
					event_handler = EventHandler,
					modified_attributes = ModifiedAttributes,
					has_been_checked = HasBeenChecked,
					should_be_scheduled = ShouldBeScheduled
				where id = thisHostID;
		end if;

		-- update these values about the host-check
		-- (or insert the check if it is missing)
		c := count(*) from host_check where hostID = thisHostID;

		if c = 0
		then
			insert into host_check (check_command,
						check_execute_time,
						check_latency,
						last_check,
						next_check,
						max_attempts,
						last_notification,
						next_notification,
						no_more_notifications,
						performance_data,
						check_type,
						hostID)
					values (CheckCommand,
						CheckExecuteTime,
						CheckLatency,
						LastCheck,
						NextCheck,
						MaxAttempts,
						LastNotification,
						NextNotification,
						NoMoreNotifications,
						PerformanceData,
						CheckType,
						thisHostID);
		else	
			update host_check set check_command = CheckCommand,
					      check_execute_time = CheckExecuteTime,
					      check_latency = CheckLatency,
					      last_check = LastCheck,
					      next_check = NextCheck,
					      max_attempts = MaxAttempts,
					      last_notification = LastNotification,
					      next_notification = NextNotification,
					      no_more_notifications = NoMoreNotifications,
					      performance_data = PerformanceData,
					      check_type = CheckType
				where hostID = thisHostID;
		end if;

		-- pull this machine in or out of flapping_host based on is_flapping
		-- It will help if we know if we were flapping before this execution
		c := count(*) from flapping_host where id = thisHostID;

		if IsFlapping = TRUE and c = 0
		then
			-- we are flapping and we were not before
			insert into flapping_host (id) values (thisHostID);
		else
			if IsFlapping = FALSE and c = 1
			then
				-- we were flapping and we are not anymore
				delete from flapping_host where id = thisHostID;
			end if;
		end if;

		-- Finally, either extend the current service period or make a new one.
		-- We will need to look up the current soft_state and hard_state.
		select into states soft_state,hard_state FROM host_history WHERE hostid = thisHostID and lasted is null;

		-- If they match current_state and last_hard_state, look like we extend.
		if states.soft_state = SoftState and states.hard_state = HardState
		then
			-- Extending means do nothing, unless we are in a soft bad state
			if states.soft_state > 0 -- non-zero means non-good
			then
				-- we need to update:
				-- current_attempt
				update host_history set current_attempt = CurrentAttempt where hostid = thisHostID and lasted is null;
			end if;
		else
			-- Otherwise, we end the existing period and start a new one.....
			update host_history set lasted = now()::abstime::int4 - started where hostid = thisHostID and lasted is null;
			insert into host_history (started,soft_state,hard_state,plugin_output,hostid,current_attempt) values
						(now()::abstime::int4,SoftState,HardState,PluginOutput,thisHostID,CurrentAttempt);
		end if;

		return 0;
        END;
' LANGUAGE 'plpgsql';
