create table host
(
	id serial primary key, -- the unique numeric id for this host
	name text unique not null, -- the name of this host; how nagios keeps things unique
	active_checks_enabled boolean not null, 
	passive_checks_enabled boolean not null,
	event_handler_enabled boolean not null,
	flap_detection_enabled boolean not null,
	notifications_enabled boolean not null,
	failure_prediction_enabled boolean not null, 
	process_performance_data boolean not null,
	obsess_over_host boolean not null,
	event_handler text, -- name of the event handler command, if any
	
	-- these should be irrelevent for us in terms of computation, but we gather them anyway as they might help in UI presentation
	modified_attributes int, -- a bitfield of flags
	has_been_checked boolean not null, -- not sure when this would ever be false, except perhaps for new things
	should_be_scheduled boolean not null -- Does this just refer to the host check, or to all services on this host?
);
create index index_host_name on host(name);

create table service
(
	id serial primary key, -- the unique numeric id for this service; known only to the db
	hostid int not null references host(id), -- the id of the host the service is attached to
	service_description text not null, -- the description of the service; combined with the host, how nagios keeps services unique
	active_checks_enabled boolean not null, 
	passive_checks_enabled boolean not null,
	event_handler_enabled boolean not null,
	flap_detection_enabled boolean not null,
	notifications_enabled boolean not null,
	failure_prediction_enabled boolean not null, 
	process_performance_data boolean not null,
	obsess_over_service boolean not null,
	event_handler text, -- name of the event handler command, if any
	
	-- these should be irrelevent for us in terms of computation, but we gather them anyway as they might help in UI presentation
	modified_attributes int, -- a bitfield of flags
	has_been_checked boolean not null, -- not sure when this would ever be false, except perhaps for new things
	should_be_scheduled boolean not null -- synonymous with "is this service active"?
);
create index index_service_name on service (hostid, service_description);

create table host_check
(
	hostid int not null references host(id),
	check_command text not null, -- what command is used to check host availability
	check_execute_time float not null, -- execute time of said command
	check_latency float not null, -- how late the check was in running
	last_check int not null, -- unixtime of last check
	next_check int not null, -- unixtime of next scheduled check
	max_attempts int not null, -- how many times this will be tried
	last_notification int not null, -- unixtime for the last notification
	next_notification int not null, -- unixtime for the next notification
	no_more_notifications boolean not null, -- will more notifications be sent?
	check_type int not null,

	-- not sure about these, but what the hell
	performance_data text, 
	current_notification_number int
);
create index host_check_hostid_index on host_check(hostid);


create table service_check
(
	serviceid int,
	check_command text, -- what command is used to check host availability
	check_execute_time float, -- execute time of said command
	check_latency float, -- how late the check was in running
	last_check int, -- unixtime of last check
	next_check int, -- unixtime of next scheduled check
	max_attempts int, -- how many times this will be tried
	last_notification int, -- unixtime for the last notification
	next_notification int, -- unixtime for the next notification
	no_more_notifications boolean, -- will more notifications be sent?
	check_type int not null,

	-- not sure about these, but what the hell
	performance_data text,
	current_notification_number int	
);
create index service_check_serviceid_index on service_check(serviceid);

create table host_history
(
	hostid int not null references host(id),
	started int not null, --unixtime of when the host entered this period
	lasted int, --seconds the host remained in this period
	soft_state int, --pulled from "current_state"
	hard_state int, --pulled from "last_hard_state" 
	current_attempt int, -- how many times this has been tried? 
	plugin_output text -- the plugin output at the start of this period
);

create table service_history
(
	serviceid int not null references service(id),
	started int not null, --unixtime of when the service entered this period
	lasted int, --seconds the service remained in this period
	soft_state int, --pulled from "current_state"
	hard_state int, --pulled from "last_hard_state" 
	current_attempt int, -- how many times this has been tried? 
	plugin_output text -- the plugin output at the start of this period
);

-- the entries below are pulled from the "is_flapping" fields in the status events
create table flapping_host
(
	id int not null references host(id) primary key
);

create table flapping_service
(
	id int not null references service(id) primary key
);

-- for the materialzed views
CREATE TABLE matviews (
  mv_name NAME NOT NULL PRIMARY KEY
  , v_name NAME NOT NULL
  , last_refresh TIMESTAMP WITH TIME ZONE
);

