create or replace view host_perf_data as
	select 
		min(check_execute_time) as min_execute_time,
		max(check_execute_time) as max_execute_time,
		avg(check_execute_time) as avg_execute_time,
		min(check_latency) as min_latency,
		max(check_latency) as max_latency,
		avg(check_latency) as avg_latency
	from 
		host_check;


create or replace view service_perf_data as
	select 
		min(check_execute_time) as min_execute_time,
		max(check_execute_time) as max_execute_time,
		avg(check_execute_time) as avg_execute_time,
		min(check_latency) as min_latency,
		max(check_latency) as max_latency,
		avg(check_latency) as avg_latency
	from 
		service_check;

create or replace view check_counts as
	select 
		s.check_type,
		s.c as services,
		h.c as hosts 
	from 
		(
			select 
				count(*) as c,
				check_type 
			from 
				service,
				service_check
			where
				service.id = service_check.serviceid
			group by check_type
		) s, 
		(
			select 
				count(*) as c,
				check_type 
			from 
				host,
				host_check
			where
				host.id = host_check.hostid
			group by check_type
		) h
	where
		s.check_type = h.check_type;

create or replace view host_states as
	select
		count(*),
		soft_state
	from
		host_history,
		host_check 
	where
		host_history.hostid = host_check.hostid and 
		last_check > 0 and 
		lasted is null
	group by soft_state 
	union 
	select 
		count(*),
		-1 
	from 
		host_check 
	where 
		last_check = 0;


create or replace view service_states as
	select 
		good.state as state,
		coalesce(good.c,0) + coalesce(bad.c,0) as total,
		coalesce(good.c,0) as good_hosts,
		coalesce(bad.c,0) as bad_hosts
	from
		(
			select 
				count(*) as c,
				service_history.soft_state as state
			from 
				service_history,
				service_check,
				host_history,
				service
			where 
				host_history.lasted is null and
				host_history.soft_state = 0 and
				host_history.hostid = service.hostid and
				service_history.serviceid = service.id and
				service_history.serviceid = service_check.serviceid and 
				service_check.last_check > 0 and 
				service_history.lasted is null 
			group by state 
			union 
			select 
				count(*),
				-1 
			from 
				service_check,
				service,
				host_history
			where 
				service.id = service_check.serviceid and
				host_history.hostid = service.hostid and
				host_history.lasted is null and
				host_history.soft_state = 0 and
				service_check.last_check = 0
		) good full outer join 
		(
			select 
				count(*) as c,
				service_history.soft_state as state
			from 
				service_history,
				service_check,
				host_history,
				service
			where 
				host_history.lasted is null and
				host_history.soft_state != 0 and
				host_history.hostid = service.hostid and
				service_history.serviceid = service.id and
				service_history.serviceid = service_check.serviceid and 
				service_check.last_check > 0 and 
				service_history.lasted is null 
			group by state 
			union 
			select 
				count(*),
				-1 
			from 
				service_check,
				service,
				host_history
			where 
				service.id = service_check.serviceid and
				host_history.hostid = service.hostid and
				host_history.lasted is null and
				host_history.soft_state != 0 and
				service_check.last_check = 0
		) bad
	on
		bad.state = good.state;


create or replace view option_exceptions as
	select
		'host' as type,
		sum(case active_checks_enabled when FALSE then 1 else 0 end) as active,
		sum(case passive_checks_enabled when FALSE then 1 else 0 end) as passive,
		sum(case notifications_enabled when FALSE then 1 else 0 end) as notification,
		sum(case flap_detection_enabled when FALSE then 1 else 0 end) as flap,
		sum(case event_handler_enabled when FALSE then 1 else 0 end) as event
	from
		host
	union
	select
		'service' as type,
		sum(case active_checks_enabled when FALSE then 1 else 0 end) as active,
		sum(case passive_checks_enabled when FALSE then 1 else 0 end) as passive,
		sum(case notifications_enabled when FALSE then 1 else 0 end) as notification,
		sum(case flap_detection_enabled when FALSE then 1 else 0 end) as flap,
		sum(case event_handler_enabled when FALSE then 1 else 0 end) as event
	from
		service;


select create_matview('check_counts_mv','check_counts');
select create_matview('host_perf_data_mv','host_perf_data');
select create_matview('host_states_mv','host_states');
select create_matview('option_exceptions_mv','option_exceptions');
select create_matview('service_perf_data_mv','service_perf_data');
select create_matview('service_states_mv','service_states');

