/*****************************************************************************
 *
 * EXTHANDLER.C - NEB module that logs to a socket based on client requests
 *
 * Copyright (c) 2004 Titus Anderson (titus.anderson@louisville.edu)
 *
 * Last Modified:   09-30-2004
 *
 * Description:
 *
 *
 * Not yet completed, but it works. Opens the file given as an argument to the
 * handler. Clients connect to the socket and write a string +XXX or -XXX to
 * indicate which events it's interested in. The events are periodically
 * (currently every second) sent to the client in the order in which they are
 * recieved. The format of an event is the event type, followed by the
 * attributes of the that event type, and a closing line, all on separate
 * lines. The module is able to handle multiple clients all requesting any
 * number of different event types.
 *
 * Instructions:
 *
 * Compile with the following command:
 *
 *     gcc -shared -o exthandler.o exthandler.c
 *
 *****************************************************************************/

#ifndef NSCORE
#define NSCORE
#endif

/* we need this for postgres support */
#include "libpq-fe.h"

/* include (minimum required) event broker header files */
#include "../include/nebmodules.h"
#include "../include/nebcallbacks.h"
#include "../include/nebstructs.h"

/* include some Nagios stuff as well */
#include "../include/config.h"
#include "../include/common.h"
#include "../include/nagios.h"
#include "../include/objects.h"

#include <sys/types.h>
#include <string.h>
#include <stdlib.h>

#define UNIX_PATH_MAX 108

/* specify event broker API version (required) */
NEB_API_VERSION(CURRENT_NEB_API_VERSION);

#define BUFFER_ALLOC_SIZE 1024
#define NULL_STRING(x) (x?:"null")
#define BOOLIFY_STRING(x) (x?"TRUE":"FALSE")

#define FIRST_COMMAND_NUM NEBCALLBACK_PROCESS_DATA


static int processCallback(int, void *);


PGconn * pgconn = 0;

/* this function gets called when the module is loaded by the event broker */
int nebmodule_init(int flags, char *args, nebmodule *handle)
{
	write_to_all_logs("initializing inserter",NSLOG_INFO_MESSAGE);


	if(!(pgconn = PQconnectdb("host=dumont port=5432 dbname=nag user=postgres")))
	{
		// couldn't connect; this will be useless.
		write_to_all_logs("inserter failed to connect to db",NSLOG_INFO_MESSAGE);
	}

	write_to_all_logs("inserter locked and loaded",NSLOG_INFO_MESSAGE);

	neb_register_callback(NEBCALLBACK_HOST_STATUS_DATA, 0, processCallback);
	neb_register_callback(NEBCALLBACK_SERVICE_STATUS_DATA, 0, processCallback);

	return 0;
}


/* this function gets called when the module is unoaded by the event broker */
int nebmodule_deinit(int flags, int reason){
	
	/* log a message to the Nagios log file */

	write_to_all_logs("inserter unloading...",NSLOG_INFO_MESSAGE);

	PQfinish(pgconn);

	neb_deregister_callback(NEBCALLBACK_HOST_STATUS_DATA, processCallback);
	neb_deregister_callback(NEBCALLBACK_SERVICE_STATUS_DATA, processCallback);

	pgconn = 0;

	return 0;
}

char *
querify(char * src)
{
	char * ret = 0;
	if(src)
	{
		int size = 2*strlen(src) + 2; // we want to surround strings with single quotes
		int written = 0;
		if(!(ret = malloc(size)))
		{
			// problem
			return 0;
		}
		ret[0] = '\'';
		written = PQescapeString(ret+1, src, size);
		ret[written + 1] = '\'';
		ret[written + 2] = 0;
	}

	return ret;
}

static int processCallback(int cmd, void * data) {
	PGresult	*res;
 	char		temp_buffer[5120];

	service * tmp_service;
	host * tmp_host;

	bzero(temp_buffer, sizeof(temp_buffer));
	switch(cmd) 
	{
		case NEBCALLBACK_HOST_STATUS_DATA:
		{
			nebstruct_host_status_data * host_status_data = (nebstruct_host_status_data *) data;
			tmp_host = (host *) host_status_data->object_ptr;
			char *safeName = querify(tmp_host->name);
			char *safeEventHandler = querify(tmp_host->event_handler);
			char *safePluginOutput = querify(tmp_host->plugin_output);
			char *safeHostCheckCommand = querify(tmp_host->host_check_command);
			char *safePerfData = querify(tmp_host->perf_data);


			snprintf(temp_buffer, sizeof(temp_buffer), "select update_host(%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%d,%s,%s,%s,%d,%d,%s,%d,%s,%f,%f,%d,%d,%d,%d,%d,%s,%s,%d)",
														NULL_STRING(safeName),
														BOOLIFY_STRING(tmp_host->checks_enabled),
														BOOLIFY_STRING(tmp_host->accept_passive_host_checks),
														BOOLIFY_STRING(tmp_host->event_handler_enabled),
														BOOLIFY_STRING(tmp_host->flap_detection_enabled),
														BOOLIFY_STRING(tmp_host->notifications_enabled),
														BOOLIFY_STRING(tmp_host->failure_prediction_enabled),
														BOOLIFY_STRING(tmp_host->process_performance_data),
														BOOLIFY_STRING(tmp_host->obsess_over_host),
														NULL_STRING(safeEventHandler),
														tmp_host->modified_attributes,
														BOOLIFY_STRING(tmp_host->has_been_checked),
														BOOLIFY_STRING(tmp_host->should_be_scheduled),
														BOOLIFY_STRING(tmp_host->is_flapping),
														tmp_host->current_state,
														tmp_host->last_hard_state,
														NULL_STRING(safePluginOutput),
														tmp_host->current_attempt,
														NULL_STRING(safeHostCheckCommand),
														tmp_host->execution_time,
														tmp_host->latency,
														tmp_host->last_check,
														tmp_host->next_check,
														tmp_host->max_attempts,
														tmp_host->last_host_notification,
														tmp_host->next_host_notification,
														BOOLIFY_STRING(tmp_host->no_more_notifications),
														NULL_STRING(safePerfData),
														tmp_host->check_type);

			if(safeName) free(safeName);
			if(safeEventHandler) free(safeEventHandler);
			if(safePluginOutput) free(safePluginOutput);
			if(safeHostCheckCommand) free(safeHostCheckCommand);
			if(safePerfData) free(safePerfData);
		}
		break;

		case NEBCALLBACK_SERVICE_STATUS_DATA:
		{
			nebstruct_service_status_data * service_status_data = (nebstruct_service_status_data *) data;
			tmp_service = (service *) service_status_data->object_ptr;

			char *safeName = querify(tmp_service->host_name);
			char *safeServiceDescription = querify(tmp_service->description);
			char *safeEventHandler = querify(tmp_service->event_handler);
			char *safePluginOutput = querify(tmp_service->plugin_output);
			char *safeServiceCheckCommand = querify(tmp_service->service_check_command);
			char *safePerfData = querify(tmp_service->perf_data);


			snprintf(temp_buffer, sizeof(temp_buffer), "select update_service(%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%d,%s,%s,%s,%d,%d,%s,%d,%s,%f,%f,%d,%d,%d,%d,%d,%s,%s,%d)",
														NULL_STRING(safeName),
														NULL_STRING(safeServiceDescription),
														BOOLIFY_STRING(tmp_service->checks_enabled),
														BOOLIFY_STRING(tmp_service->accept_passive_service_checks),
														BOOLIFY_STRING(tmp_service->event_handler_enabled),
														BOOLIFY_STRING(tmp_service->flap_detection_enabled),
														BOOLIFY_STRING(tmp_service->notifications_enabled),
														BOOLIFY_STRING(tmp_service->failure_prediction_enabled),
														BOOLIFY_STRING(tmp_service->process_performance_data),
														BOOLIFY_STRING(tmp_service->obsess_over_service),
														NULL_STRING(safeEventHandler),
														tmp_service->modified_attributes,
														BOOLIFY_STRING(tmp_service->has_been_checked),
														BOOLIFY_STRING(tmp_service->should_be_scheduled),
														BOOLIFY_STRING(tmp_service->is_flapping),
														tmp_service->current_state,
														tmp_service->last_hard_state,
														NULL_STRING(safePluginOutput),
														tmp_service->current_attempt,
														NULL_STRING(safeServiceCheckCommand),
														tmp_service->execution_time,
														tmp_service->latency,
														tmp_service->last_check,
														tmp_service->next_check,
														tmp_service->max_attempts,
														tmp_service->last_notification,
														tmp_service->next_notification,
														BOOLIFY_STRING(tmp_service->no_more_notifications),
														NULL_STRING(safePerfData),
														tmp_service->check_type);

			if(safeName) free(safeName);
			if(safeServiceDescription) free(safeServiceDescription);
			if(safeEventHandler) free(safeEventHandler);
			if(safePluginOutput) free(safePluginOutput);
			if(safeServiceCheckCommand) free(safeServiceCheckCommand);
			if(safePerfData) free(safePerfData);
		}
    	break;

		default:
			snprintf(temp_buffer,sizeof(temp_buffer),"ignoring unknown NEB command %i",cmd);
    		write_to_all_logs(temp_buffer,NSLOG_INFO_MESSAGE);
		break;
	}

	res = PQexec(pgconn,temp_buffer);

	if(atoi(PQgetvalue(res,0,0)) < 0)
	{
		write_to_all_logs("inserter stored proc failed",NSLOG_INFO_MESSAGE);
	}

	PQclear(res);

	return 0;
}

