/*****************************************************************************
 *
 * EXTHANDLER.C - NEB module that logs to a socket based on client requests
 *
 * Copyright (c) 2004 Titus Anderson (titus.anderson@louisville.edu)
 *
 * Last Modified:   09-30-2004
 *
 * Description:
 *
 *
 * Not yet completed, but it works. Opens the file given as an argument to the
 * handler. Clients connect to the socket and write a string +XXX or -XXX to
 * indicate which events it's interested in. The events are periodically
 * (currently every second) sent to the client in the order in which they are
 * recieved. The format of an event is the event type, followed by the
 * attributes of the that event type, and a closing line, all on separate
 * lines. The module is able to handle multiple clients all requesting any
 * number of different event types.
 *
 * Instructions:
 *
 * Compile with the following command:
 *
 *     gcc -shared -o exthandler.o exthandler.c
 *
 *****************************************************************************/

#ifndef NSCORE
#define NSCORE
#endif

/* include (minimum required) event broker header files */
#include "../include/nebmodules.h"
#include "../include/nebcallbacks.h"
#include "../include/nebstructs.h"

/* include some Nagios stuff as well */
#include "../include/config.h"
#include "../include/common.h"
#include "../include/nagios.h"
#include "../include/objects.h"

#include <sys/types.h>
#include <string.h>
#include <stdlib.h>

#define UNIX_PATH_MAX 108

/* specify event broker API version (required) */
NEB_API_VERSION(CURRENT_NEB_API_VERSION);

#define BUFFER_ALLOC_SIZE 1024
#define NULL_STRING(x) (x?x:"")
#define FLAG_CHR(flag, chr) (flag?chr:(chr-'A'+'a'))

static char * commands[] = {
  "PROCESS_DATA",
  NULL,
  "LOG",
  "SYSTEM_COMMAND",
  "EVENT_HANDLER",
  "NOTIFICATION",
  "SERVICE_CHECK",
  "HOST_CHECK",
  "COMMENT",
  "DOWNTIME",
  "FLAPPING",
  "PROGRAM_STATUS",
  "HOST_STATUS",
  "SERVICE_STATUS"
};

#define FIRST_COMMAND_NUM NEBCALLBACK_PROCESS_DATA
static int numCommands = sizeof(commands) / sizeof(char *);


static int processCallback(int, void *);

typedef struct message_struct {
  char * messageData;
  int refCount;
} message;

typedef struct message_list_struct {
  message * msg;
  struct message_list_struct * next;
} message_list;

typedef struct client_struct {
  int clientSocket;
  unsigned int registrations;
  message_list * messages;
  message_list * messagetail;
  int defunct:1;
  struct client_struct * next;
} client;

static client * client_list = NULL;

FILE * filep = 0;

/* this function gets called when the module is loaded by the event broker */
int nebmodule_init(int flags, char *args, nebmodule *handle){
	time_t current_time;
	unsigned long interval;

	filep = fopen(args, "w");

	write_to_all_logs("exthandler loaded",NSLOG_INFO_MESSAGE);
        printf("LOADING...\n");

	time(&current_time);
	interval=1;
	schedule_new_event(EVENT_USER_FUNCTION,TRUE,current_time+interval,TRUE,interval,NULL,TRUE,NULL,NULL);

	neb_register_callback(NEBCALLBACK_PROCESS_DATA, 0, processCallback);
	/*neb_register_callback(NEBCALLBACK_TIMED_EVENT_DATA, 0, processCallback); */
	neb_register_callback(NEBCALLBACK_LOG_DATA, 0, processCallback);
	neb_register_callback(NEBCALLBACK_SYSTEM_COMMAND_DATA, 0, processCallback);
	neb_register_callback(NEBCALLBACK_EVENT_HANDLER_DATA, 0, processCallback);
	neb_register_callback(NEBCALLBACK_NOTIFICATION_DATA, 0, processCallback);
	neb_register_callback(NEBCALLBACK_SERVICE_CHECK_DATA, 0, processCallback);
	neb_register_callback(NEBCALLBACK_HOST_CHECK_DATA, 0, processCallback);
	neb_register_callback(NEBCALLBACK_COMMENT_DATA, 0, processCallback);
	neb_register_callback(NEBCALLBACK_DOWNTIME_DATA, 0, processCallback);
	neb_register_callback(NEBCALLBACK_FLAPPING_DATA, 0, processCallback);
	neb_register_callback(NEBCALLBACK_PROGRAM_STATUS_DATA, 0, processCallback);
	neb_register_callback(NEBCALLBACK_HOST_STATUS_DATA, 0, processCallback);
	neb_register_callback(NEBCALLBACK_SERVICE_STATUS_DATA, 0, processCallback);
	return 0;
        }


/* this function gets called when the module is unoaded by the event broker */
int nebmodule_deinit(int flags, int reason){
	char temp_buffer[1024];
	
	/* log a message to the Nagios log file */
	snprintf(temp_buffer,sizeof(temp_buffer)-1,"exthandler unloading\n");
	temp_buffer[sizeof(temp_buffer)-1]='\x0';
	write_to_all_logs(temp_buffer,NSLOG_INFO_MESSAGE);

	neb_deregister_callback(NEBCALLBACK_PROCESS_DATA, processCallback);
	/*neb_deregister_callback(NEBCALLBACK_TIMED_EVENT_DATA, processCallback);*/
	neb_deregister_callback(NEBCALLBACK_LOG_DATA, processCallback);
	neb_deregister_callback(NEBCALLBACK_SYSTEM_COMMAND_DATA, processCallback);
	neb_deregister_callback(NEBCALLBACK_EVENT_HANDLER_DATA, processCallback);
	neb_deregister_callback(NEBCALLBACK_NOTIFICATION_DATA, processCallback);
	neb_deregister_callback(NEBCALLBACK_SERVICE_CHECK_DATA, processCallback);
	neb_deregister_callback(NEBCALLBACK_HOST_CHECK_DATA, processCallback);
	neb_deregister_callback(NEBCALLBACK_COMMENT_DATA, processCallback);
	neb_deregister_callback(NEBCALLBACK_DOWNTIME_DATA, processCallback);
	neb_deregister_callback(NEBCALLBACK_FLAPPING_DATA, processCallback);
	neb_deregister_callback(NEBCALLBACK_PROGRAM_STATUS_DATA, processCallback);
	neb_deregister_callback(NEBCALLBACK_HOST_STATUS_DATA, processCallback);
	neb_deregister_callback(NEBCALLBACK_SERVICE_STATUS_DATA, processCallback);

	fclose(filep);
	filep = 0;

	return 0;
        }

static message * newMessage(const char * text) {
  message * msg;

  msg = (message *) malloc(sizeof(message));
  msg->messageData = (char *) strdup(text);
  msg->refCount = 0;

  /* printf("New message %p\n", msg);*/

  return msg;
}

void freeMessage(message * msg) {
  msg->refCount--;
  /*printf("\t\t\t\t\t\t%i\n", msg->refCount);*/
  if (msg->refCount <= 0) {
    /*printf("Freeing message %p\n", msg);*/
    if (msg->messageData) free(msg->messageData);
    free(msg);
  }
}

static int writeToClient(client * c, message * m) {
  message_list * newitem;

  m->refCount++;

  newitem = (message_list *) malloc(sizeof(message_list));
  newitem->msg = m;
  newitem->next = NULL;

  if (!c->messages) {
    c->messages = newitem;
  } else {
    c->messagetail->next = newitem;
  }

  c->messagetail = newitem;

  return 0;
}

static void flushMessageList(client * c) {
  message_list * p, * q;

  for (p = q = c->messages; p != NULL; q = p, p = p->next) {
    freeMessage(p->msg);
    free(p);
  }

  c->messagetail = c->messages = NULL;
}

static int processCallback(int cmd, void * data) {
  char temp_buffer[2048];
  client * c;
  nebstruct_service_check_data * service_check_data;
  nebstruct_host_check_data * host_check_data;
  nebstruct_service_status_data * service_status_data;
  nebstruct_host_status_data * host_status_data;
  nebstruct_notification_data * notification_data;
  nebstruct_process_data * process_data;
  nebstruct_log_data * log_data;
  nebstruct_system_command_data * system_command_data;
  nebstruct_event_handler_data * event_handler_data;
  nebstruct_comment_data * comment_data;
  nebstruct_downtime_data * downtime_data;
  nebstruct_flapping_data * flapping_data;
  nebstruct_program_status_data * program_status_data;

  message * msg;

  service * tmp_service;
  host * tmp_host;

  /*printf("CMD IS %i\n", cmd);*/

  bzero(temp_buffer, sizeof(temp_buffer));
  switch(cmd) {
  case NEBCALLBACK_SERVICE_CHECK_DATA:
    service_check_data = (nebstruct_service_check_data *) data;
    sprintf(temp_buffer, "SERVICE_CHECK "
                         "type=%s "
                         "host_name=%s "
                         "service_description=%s "
                         "check_type=%i "
                         "current_attempt=%i "
                         "max_attempts=%i "
                         "state_type=%i "
                         "state=%i "
                         "timeout=%i "
                         "command_line=%s "
                         "early_timeout=%i "
                         "execution_time=%f "
                         "latency=%f "
                         "return_code=%i "
                         "output=%s "
                         "perf_data=%s "
                         "END_SERVICE_CHECK\n", 
            (service_check_data->type == 700)?"INIT":
            (service_check_data->type == 702)?"FINAL":"UNKNOWN",
            NULL_STRING(service_check_data->host_name),
            NULL_STRING(service_check_data->service_description),
            service_check_data->check_type,
            service_check_data->current_attempt,
            service_check_data->max_attempts,
            service_check_data->state_type,
            service_check_data->state,
            service_check_data->timeout,
            NULL_STRING(service_check_data->command_line),
            service_check_data->early_timeout,
            service_check_data->execution_time,
            service_check_data->latency,
            service_check_data->return_code,
            NULL_STRING(service_check_data->output),
            NULL_STRING(service_check_data->perf_data));
    break;
  case NEBCALLBACK_HOST_CHECK_DATA:
    host_check_data = (nebstruct_host_check_data *) data;
    sprintf(temp_buffer, "HOST_CHECK "
                         "type=%s "
                         "host_name=%s "
                         "check_type=%i "
                         "current_attempt=%i "
                         "max_attempts=%i "
                         "state_type=%i "
                         "state=%i "
                         "timeout=%i "
                         "command_line=%s "
                         "early_timeout=%i "
                         "execution_time=%f "
                         "latency=%f "
                         "return_code=%i "
                         "output=%s "
                         "perf_data=%s "
                         "END_HOST_CHECK\n", 
            (host_check_data->type == 800)?"INIT":
            (host_check_data->type == 801)?"RAW":
            (host_check_data->type == 802)?"FINAL":"UNKNOWN",
            NULL_STRING(host_check_data->host_name),
            host_check_data->check_type,
            host_check_data->current_attempt,
            host_check_data->max_attempts,
            host_check_data->state_type,
            host_check_data->state,
            host_check_data->timeout,
            NULL_STRING(host_check_data->command_line),
            host_check_data->early_timeout,
            host_check_data->execution_time,
            host_check_data->latency,
            host_check_data->return_code,
            NULL_STRING(host_check_data->output),
            NULL_STRING(host_check_data->perf_data));
    break;
  case NEBCALLBACK_HOST_STATUS_DATA:
    host_status_data = (nebstruct_host_status_data *) data;
    tmp_host = (host *) host_status_data->object_ptr;
    sprintf(temp_buffer, "HOST_STATUS "
                         "host_name=%s "
                         "modified_attributes=%lu "
                         "check_command=%s "
                         "event_handler=%s "
                         "has_been_checked=%d "
                         "should_be_scheduled=%d "
                         "check_execution_time=%.3f "
                         "check_latency=%.3f "
                         "current_state=%d "
                         "last_hard_state=%d "
                         "check_type=%d "
                         "plugin_output=%s "
                         "performance_data=%s "
                         "last_check=%lu "
                         "next_check=%lu "
                         "current_attempt=%d "
                         "max_attempts=%d "
                         "state_type=%d "
                         "last_state_change=%lu "
                         "last_hard_state_change=%lu "
                         "last_time_up=%lu "
                         "last_time_down=%lu "
                         "last_time_unreachable=%lu "
                         "last_notification=%lu "
                         "next_notification=%lu "
                         "no_more_notifications=%d "
                         "current_notification_number=%d "
                         "notifications_enabled=%d "
                         "problem_has_been_acknowledged=%d "
                         "acknowledgement_type=%d "
                         "active_checks_enabled=%d "
                         "passive_checks_enabled=%d "
                         "event_handler_enabled=%d "
                         "flap_detection_enabled=%d "
                         "failure_prediction_enabled=%d "
                         "process_performance_data=%d "
                         "obsess_over_host=%d "
                         "is_flapping=%d "
                         "percent_state_change=%.2f "
                         "scheduled_downtime_depth=%d "
                         "END_HOST_STATUS\n",
             tmp_host->name,
             tmp_host->modified_attributes,
             NULL_STRING(tmp_host->host_check_command),
             NULL_STRING(tmp_host->event_handler),
             tmp_host->has_been_checked,
             tmp_host->should_be_scheduled,
             tmp_host->execution_time,
             tmp_host->latency,
             tmp_host->current_state,
             tmp_host->last_hard_state,
             tmp_host->check_type,
             NULL_STRING(tmp_host->plugin_output),
             NULL_STRING(tmp_host->perf_data),
             tmp_host->last_check,
             tmp_host->next_check,
             tmp_host->current_attempt,
             tmp_host->max_attempts,
             tmp_host->state_type,
             tmp_host->last_state_change,
             tmp_host->last_hard_state_change,
             tmp_host->last_time_up,
             tmp_host->last_time_down,
             tmp_host->last_time_unreachable,
             tmp_host->last_host_notification,
             tmp_host->next_host_notification,
             tmp_host->no_more_notifications,
             tmp_host->current_notification_number,
             tmp_host->notifications_enabled,
             tmp_host->problem_has_been_acknowledged,
             tmp_host->acknowledgement_type,
             tmp_host->checks_enabled,
             tmp_host->accept_passive_host_checks,
             tmp_host->event_handler_enabled,
             tmp_host->flap_detection_enabled,
             tmp_host->failure_prediction_enabled,
             tmp_host->process_performance_data,
             tmp_host->obsess_over_host,
             tmp_host->is_flapping,
             tmp_host->percent_state_change,
             tmp_host->scheduled_downtime_depth);
    break;
  case NEBCALLBACK_SERVICE_STATUS_DATA:
    service_status_data = (nebstruct_service_status_data *) data;
    tmp_service = (service *) service_status_data->object_ptr;
    /* This has a full service description so it's LONG!!! */
sprintf(temp_buffer, "SERVICE_STATUS "
                     "host_name=%s "
                     "service_description=%s "
                     "modified_attributes=%lu "
                     "check_command=%s "
                     "event_handler=%s "
                     "has_been_checked=%d "
                     "should_be_scheduled=%d "
                     "check_execution_time=%.3f "
                     "check_latency=%.3f "
                     "current_state=%d "
                     "last_hard_state=%d "
                     "current_attempt=%d "
                     "max_attempts=%d "
                     "state_type=%d "
                     "last_state_change=%lu "
                     "last_hard_state_change=%lu "
                     "last_time_ok=%lu "
                     "last_time_warning=%lu "
                     "last_time_unknown=%lu "
                     "last_time_critical=%lu "
                     "plugin_output=%s "
                     "performance_data=%s "
                     "last_check=%lu "
                     "next_check=%lu "
                     "check_type=%d "
                     "current_notification_number=%d "
                     "last_notification=%lu "
                     "next_notification=%lu "
                     "no_more_notifications=%d "
                     "notifications_enabled=%d "
                     "active_checks_enabled=%d "
                     "passive_checks_enabled=%d "
                     "event_handler_enabled=%d "
                     "problem_has_been_acknowledged=%d "
                     "acknowledgement_type=%d "
                     "flap_detection_enabled=%d "
                     "failure_prediction_enabled=%d "
                     "process_performance_data=%d "
                     "obsess_over_service=%d "
                     "is_flapping=%d "
                     "percent_state_change=%.2f "
                     "scheduled_downtime_depth=%d "
                     "END_SERVICE_STATUS\n",
        tmp_service->host_name,
        tmp_service->description,
        tmp_service->modified_attributes,
        NULL_STRING(tmp_service->service_check_command),
        NULL_STRING(tmp_service->event_handler),
        tmp_service->has_been_checked,
        tmp_service->should_be_scheduled,
        tmp_service->execution_time,
        tmp_service->latency,
        tmp_service->current_state,
        tmp_service->last_hard_state,
        tmp_service->current_attempt,
        tmp_service->max_attempts,
        tmp_service->state_type,
        tmp_service->last_state_change,
        tmp_service->last_hard_state_change,
        tmp_service->last_time_ok,
        tmp_service->last_time_warning,
        tmp_service->last_time_unknown,
        tmp_service->last_time_critical,
        NULL_STRING(tmp_service->plugin_output),
        NULL_STRING(tmp_service->perf_data),
        tmp_service->last_check,
        tmp_service->next_check,
        tmp_service->check_type,
        tmp_service->current_notification_number,
        tmp_service->last_notification,
        tmp_service->next_notification,
        tmp_service->no_more_notifications,
        tmp_service->notifications_enabled,
        tmp_service->checks_enabled,
        tmp_service->accept_passive_service_checks,
        tmp_service->event_handler_enabled,
        tmp_service->problem_has_been_acknowledged,
        tmp_service->acknowledgement_type,
        tmp_service->flap_detection_enabled,
        tmp_service->failure_prediction_enabled,
        tmp_service->process_performance_data,
        tmp_service->obsess_over_service,
        tmp_service->is_flapping,
        tmp_service->percent_state_change,
        tmp_service->scheduled_downtime_depth);
    break;
  case NEBCALLBACK_NOTIFICATION_DATA:
    notification_data = (nebstruct_notification_data *) data;
    sprintf(temp_buffer, "NOTIFICATION "
                         "type=%s "
                         "notification_type=%i "
                         "host_name=%s "
                         "service_description=%s "
                         "reason_type=%i "
                         "state=%i "
                         "output=%s "
                         "ack_author=%s "
                         "ack_data=%s "
                         "END_NOTIFICATION\n", 
            (notification_data->type==600)?"START":"END",
            notification_data->notification_type,
            NULL_STRING(notification_data->host_name),
            NULL_STRING(notification_data->service_description),
            notification_data->reason_type,
            notification_data->state,
            NULL_STRING(notification_data->output),
            NULL_STRING(notification_data->ack_author),
            NULL_STRING(notification_data->ack_data));
    break;
  case NEBCALLBACK_PROCESS_DATA:
    process_data = (nebstruct_process_data *) data;
    sprintf(temp_buffer, "PROCESS "
                         "type=%s "
                         "END_PROCESS\n",
            (process_data->type==100)?"START":
            (process_data->type==101)?"DAEMONIZE":
            (process_data->type==102)?"RESTART":"STOP");
    break;
  case NEBCALLBACK_LOG_DATA:
    log_data = (nebstruct_log_data *) data;
    sprintf(temp_buffer, "LOG "
                         "type=%s "
                         "data_type=%i "
                         "data=%s "
                         "END_LOG\n",
            (log_data->type==300)?"DATA":"ROTATION",
            log_data->data_type,
            log_data->data);
    break;
  case NEBCALLBACK_SYSTEM_COMMAND_DATA:
    system_command_data = (nebstruct_system_command_data *) data;
    sprintf(temp_buffer, "SYSTEM_COMMAND "
                         "timeout=%i "
                         "command_line=%s "
                         "early_timeout=%i "
                         "execution_time=%.2f "
                         "return_code=%i "
                         "output=%s "
                         "END_SYSTEM_COMMAND\n",
            system_command_data->timeout,
            system_command_data->command_line,
            system_command_data->early_timeout,
            system_command_data->execution_time,
            system_command_data->return_code,
            system_command_data->output);
    break;
  case NEBCALLBACK_EVENT_HANDLER_DATA:
    event_handler_data = (nebstruct_event_handler_data *) data;
    sprintf(temp_buffer, "EVENT_HANDLER "
                         "type=%s "
                         "host_name=%s "
                         "service_description=%s "
                         "state_type=%i "
                         "state=%i "
                         "timeout=%i "
                         "command_line=%s "
                         "early_timeout=%i "
                         "execution_time=%.2f "
                         "return_code=%i "
                         "output=%s "
                         "END_EVENT_HANDLER\n",
             (event_handler_data->type==500)?"HOST":
             (event_handler_data->type==501)?"GLOBAL_HOST":
             (event_handler_data->type==502)?"SERVICE":"GLOBAL_SERVICE",
             event_handler_data->host_name,
             event_handler_data->service_description,
             event_handler_data->state_type,
             event_handler_data->state,
             event_handler_data->timeout,
             event_handler_data->command_line,
             event_handler_data->early_timeout,
             event_handler_data->execution_time,
             event_handler_data->return_code,
             event_handler_data->output);
    break;
  case NEBCALLBACK_COMMENT_DATA:
    comment_data = (nebstruct_comment_data *) data;
    sprintf(temp_buffer, "COMMENT "
                         "type=%s "
                         "host_name=%s "
                         "service_description=%s "
                         "entry_time=%lu "
                         "author_name=%s "
                         "comment_data=%s "
                         "persistent=%i "
                         "source=%i "
                         "entry_type=%i "
                         "expires=%i "
                         "expire_time=%lu "
                         "comment_id=%lu "
                         "END_COMMENT\n",
             (comment_data->type==900)?"ADD":
             (comment_data->type==901)?"DELETE":"LOAD",
             comment_data->host_name,
             comment_data->service_description,
             comment_data->entry_time,
             comment_data->author_name,
             comment_data->comment_data,
             comment_data->persistent,
             comment_data->source,
             comment_data->entry_type,
             comment_data->expires,
             comment_data->expire_time,
             comment_data->comment_id);
    break;
  case NEBCALLBACK_DOWNTIME_DATA:
    downtime_data = (nebstruct_downtime_data *) data;
    sprintf(temp_buffer, "DOWNTIME "
                         "type=%s "
                         "downtime_type=%s "
                         "host_name=%s "
                         "service_description=%s "
                         "entry_time=%lu "
                         "author_name=%s "
                         "comment_data=%s "
                         "start_time=%lu "
                         "end_time=%lu "
                         "fixed=%i "
                         "duration=%lu "
                         "triggered_by=%lu "
                         "downtime_id=%lu "
                         "END_DOWNTIME\n",
             (downtime_data->type==1100)?"ADD":
             (downtime_data->type==1101)?"DELETE":
             (downtime_data->type==1102)?"LOAD":
             (downtime_data->type==1103)?"START":"STOP",
             downtime_data->downtime_type,
             downtime_data->host_name,
             downtime_data->service_description,
             downtime_data->entry_time,
             downtime_data->author_name,
             downtime_data->comment_data,
             downtime_data->start_time,
             downtime_data->end_time,
             downtime_data->fixed,
             downtime_data->duration,
             downtime_data->triggered_by,
             downtime_data->downtime_id);
    break;
  case NEBCALLBACK_FLAPPING_DATA:
    flapping_data = (nebstruct_flapping_data *) data;
    sprintf(temp_buffer, "FLAPPING "
                         "type=%s "
                         "flapping_type=%i "
                         "host_name=%s "
                         "service_description=%s "
                         "percent_change=%.2f "
                         "threshold=%.2f "
                         "comment_id=%lu "
                         "END_FLAPPING\n",
            (flapping_data->type==1000)?"START":"STOP",
            flapping_data->flapping_type,
            flapping_data->host_name,
            flapping_data->service_description,
            flapping_data->percent_change,
            flapping_data->threshold,
            flapping_data->comment_id);
    break;
  case NEBCALLBACK_PROGRAM_STATUS_DATA:
    program_status_data = (nebstruct_program_status_data *) data;
    sprintf(temp_buffer, "PROGRAM_STATUS "
                         "program_start=%lu "
                         "pid=%i "
                         "daemon_mode=%i "
                         "last_command_check=%lu "
                         "last_log_rotation=%lu "
                         "notifications_enabled=%i "
                         "active_service_checks_enabled=%i "
                         "passive_service_checks_enabled=%i "
                         "active_host_checks_enabled=%i "
                         "passive_host_checks_enabled=%i "
                         "event_handlers_enabled=%i "
                         "flap_detection_enabled=%i "
                         "failure_prediction_enabled=%i "
                         "process_performance_data=%i "
                         "obsess_over_hosts=%i "
                         "obsess_over_services=%i "
                         "END_PROGRAM_STATUS\n",
             program_status_data->program_start,
             program_status_data->pid,
             program_status_data->daemon_mode,
             program_status_data->last_command_check,
             program_status_data->last_log_rotation,
             program_status_data->notifications_enabled,
             program_status_data->active_service_checks_enabled,
             program_status_data->passive_service_checks_enabled,
             program_status_data->active_host_checks_enabled,
             program_status_data->passive_host_checks_enabled,
             program_status_data->event_handlers_enabled,
             program_status_data->flap_detection_enabled,
             program_status_data->failure_prediction_enabled,
             program_status_data->process_performance_data,
             program_status_data->obsess_over_hosts,
             program_status_data->obsess_over_services);
    break;
  default:
    sprintf(temp_buffer, "UNKNOWN cmd=%i END_UNKNOWN\n", cmd);
  }

  msg = newMessage(temp_buffer);
  msg->refCount++;

  fprintf(filep,"%lu - ",time(0));
  fprintf(filep,temp_buffer);

  for (c = client_list; c != NULL; c = c->next) {
    if (!(c->registrations & 1 << cmd)) continue;
    writeToClient(c, msg);
  }

  freeMessage(msg);

  return 0;
}

static void removeClient(client * c) {
  client * p, * q;

  if (c == NULL) return;

  c->defunct = 1;
  if (c->messages) flushMessageList(c);

  if (c == client_list) {
    client_list = client_list->next;
  } else {
    for (q = client_list; q != NULL && q->next != c; q = q->next);
    if (q == NULL) return;
    q->next = c->next;
  }

  free(c);
}

static int addClient(int s) {
  client * newClient;

  newClient = (client *) malloc(sizeof(client));
  if (!newClient) return -1;
  newClient->registrations = 0;
  newClient->defunct = 0;
  newClient->messages = newClient->messagetail = NULL;

  newClient->next = client_list;
  client_list = newClient;
}

static client * findClient(int sfd) {
  client * c = client_list;

  for (c = client_list; c != NULL; c = c->next) {
  }

  return NULL;
}

static int lookupCommand(const char * cmd) {
  int index;

  for (index = 0; index < numCommands; index++) {
    if (commands[index] == NULL) continue;
    if (!strcmp(cmd, commands[index])) return (index + FIRST_COMMAND_NUM);
  }
  return -1;
}

